/*********************************************************************
* FileName:        MRF49XA.h
* Dependencies:    See INCLUDES section below
* Processor:
* Compiler:
* Company:         Microchip Technology, Inc.
*
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
* File Description:
*
* Change History:
* Author               Cristian Toma
********************************************************************/

//#include <xc.h>
#include "./common_config.h"



#define NODATA 		0
#define DATA_RECEIVED 	1 //Means that just some of new bytes is received
#define PACKET_RECEIVED	2 //Means that the entire packet is received and can be handled

#define	PAYLOAD_MAX	10
#define PACKET_LENGTH   2
#define BYTES_SEND_DELAY_MS 25 //Depends on receiver's PWM delay (length of PWM cycle)
#define BYTES_SEND_DELAY_US 200 //For 2-bytes packets

/*#define BYTE_DEVICE_ID 0 //For 4-byte packets
#define BYTE_FUNCTION_ID 1
#define BYTE_VALUE 2
#define BYTE_CHECKSUM 3*/

#define BYTE_FUNCTION_ID 0
#define BYTE_VALUE 1

#define ZERO_VALUE_ENCODE 101
// 0: byte not received
// 1-100: usual values
// 101-109: special values
// 110-200: function codes
// 200-250: reserved

#define RF_SCK	LATCbits.LC6
#define RF_SDI	LATCbits.LC7
//#define RF_SDO	LATCbits.LC4
#define RF_SDO	PORTCbits.RC4
#define	RF_FSEL	LATDbits.LD3
#define	RF_CS	LATCbits.LC5


#define SPI_SCK RF_SCK
#define	SPI_SDI	RF_SDO
#define	SPI_SDO	RF_SDI
#define SPI_CS	RF_CS

/*#define SPI_SCK LATCbits_t.LC6
#define	SPI_SDI	LATCbits_t.LC4
#define	SPI_SDO	LATCbits_t.LC7
#define SPI_CS	LATCbits_t.LC5*/

unsigned char packet_to_send[PACKET_LENGTH];
unsigned char packet_received[PACKET_LENGTH];

unsigned char receivedBytesCount = 0;

//--------------------------------------------------------------------
// MRF49XA SPI commands:
//--------------------------------------------------------------------
const long    GENCREG 		= 0x8038;		// Cload=12.5pF; TX registers & FIFO are disabled
const long    PMCREG 		= 0x8200;		// Everything off, uC clk enabled
const long    RXCREG 		= 0x94A1;		// BW=135kHz, DRSSI=-97dBm, pin8=VDI, fast VDI
const long    TXBREG 		= 0xB800;
const long    FIFORSTREG	= 0xCA81;		// Sync. latch cleared, limit=8bits, disable sensitive reset
const long    BBFCREG 		= 0xC22C;		// Digital LPF (default)
const long    AFCCREG		= 0xC4D7;		// Auto AFC (default)
const long    CFSREG 		= 0xA7D0;		// Fo=915.000MHz (default)
const long    TXCREG		= 0x9830;		// df=60kHz, Pmax, normal modulation polarity
const long    DRSREG 		= 0xC623;		// 9579Baud (default)

//--------------------------------------------------------------------
// For F=915  MHz: lambda=32,764cm, lambda/4=9,191cm
// For F=433  MHz: lambda=68,445cm, lambda/4=17,111cm
// For F=2400 MHz: lambda=12,491cm, lambda/4=9,191cm
//--------------------------------------------------------------------
//static unsigned char is_interrupt_prohibited = 0;

void DelayMS(unsigned int msecs);
void DelayUS(unsigned int usecs);
void DelayS(unsigned int secs);
int SPI_Read(void);
unsigned char SPI_Read2(void);
void SPI_Write(int spidata);
void SPI_Write16(unsigned int spicmd);
void SPI_Command(unsigned int spicmd);
void MRF49XA_Init(void);
void MRF49XA_Send_Packet(unsigned char *data, unsigned char length);
void MRF49XA_Send_Packet2();
void MRF49XA_Send_Packet4();
int  MRF49XA_Receive_Packet(unsigned char *data, unsigned char *length);
int  MRF49XA_Receive_Packet2();
int MRF49XA_Receive_Packet4();
void MRF49XA_Power_Down(void);
void MRF49XA_Read_Status(unsigned char *ffit_rgit);
void MRF49XA_Reset_Radio(void);
void MRF49XA_Reset_Radio2(void);


//--------------------------------------------------------------------
//--------------------------------------------------------------------
//unsigned char nrbytes;
//unsigned char PacketLen;

//unsigned char packet_to_send[] = { 0x01, 0x01, 0x01, 0x01 };
//unsigned char packet_received[] = { 0x01, 0x01, 0x01, 0x01 };
