/*********************************************************************
* FileName:		   MRF49XA.c
* Dependencies:    See INCLUDES section below
* Processor:
* Compiler:
* Company:         Microchip Technology, Inc.
*
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
* File Description:
*
* Change History:
* Author               Cristian Toma
********************************************************************/
//#include <xc.h>
#include "../Headers/MRF49XA.h"
//#include "../Headers/common_config.h"



void DelayMS(unsigned int msecs)
{
    for(unsigned int i = 0; i < msecs; i++)
    {
        _delay(OPERATIONS_PER_MSEC);
    }
}

void DelayUS(unsigned int usecs)
{
    for(unsigned int i = 0; i < usecs; i++)
    {
        _delay(OPERATIONS_PER_USEC);
    }
}

void DelayS(unsigned int secs)
{
    for(unsigned long i = 0; i < (secs * 1000); i++)
    {
        _delay(OPERATIONS_PER_MSEC);
    }
}

//--------------------------------------------------------------------
// SPI_Read ()
//--------------------------------------------------------------------
int SPI_Read(void)
{
   unsigned char spidata;

   SPI_SDO = 0;
   SPI_SCK = 0;
   for (unsigned char i=0;i<8;i=i+1)
   {
       spidata=spidata<<1;

	if(SPI_SDI)
	 spidata |= 0x01;
	else
	 spidata &= 0xFE;

      SPI_SCK = 1;
      _delay(MRF49XA_SPI_DELAY);
      SPI_SCK = 0;
      _delay(MRF49XA_SPI_DELAY);
    }
    return(spidata);
}

unsigned char SPI_Read2(void)
{
   unsigned char spidata;

   SPI_SDO = 0;
   SPI_SCK = 0;
   for (unsigned char i=0;i<8;i=i+1)
   {
       spidata=spidata<<1;

	if(SPI_SDI)
	 spidata |= 0x01;
	else
	 spidata &= 0xFE;

      SPI_SCK = 1;
      _delay(MRF49XA_SPI_DELAY);
      SPI_SCK = 0;
      _delay(MRF49XA_SPI_DELAY);
    }
    return(spidata);
}
//--------------------------------------------------------------------
// SPI_Write ()
//--------------------------------------------------------------------
void SPI_Write(int spidata)
{
  char i;

  SPI_SDO = 0;
  SPI_SCK = 0;
  for (i=0;i<8;i=i+1)
    {
    if(spidata & 0x80)
        SPI_SDO = 1;
    else
        SPI_SDO = 0;

    SPI_SCK = 1;
    _delay(MRF49XA_SPI_DELAY);
    SPI_SCK = 0;
    _delay(MRF49XA_SPI_DELAY);
	spidata=spidata<<1;
    }

   SPI_SDO = 0;
}
//--------------------------------------------------------------------
// SPI_Command ()
//--------------------------------------------------------------------
void SPI_Command(unsigned int spicmd)
{
  SPI_CS = 0;
  SPI_Write ((spicmd & 0xFF00) >> 8);
  SPI_Write ((spicmd & 0x00FF));
  SPI_CS = 1;
}
//--------------------------------------------------------------------
// SPI_Write16 ()
//--------------------------------------------------------------------
void SPI_Write16(unsigned int spicmd)
{
  SPI_Write ((spicmd & 0xFF00) >> 8);
  SPI_Write ((spicmd & 0x00FF));
}
//--------------------------------------------------------------------
// FIFO syncron latch re-enable
//--------------------------------------------------------------------
void MRF49XA_Reset_Radio()
{
  SPI_Command(FIFORSTREG | 0x0002);
}
//--------------------------------------------------------------------
// FIFO syncron latch re-enable
//--------------------------------------------------------------------
void MRF49XA_Reset_Radio2()
{
	SPI_Command(PMCREG);			// turn off tx and rx
	SPI_Command(FIFORSTREG);		// reset FIFO
	SPI_Command(GENCREG);			// disable FIFO , Tx_latch
	SPI_Command(PMCREG | 0x0080);	// turn on receiver
	SPI_Command(GENCREG | 0x0040);	// enable the FIFO
	SPI_Command(FIFORSTREG | 0x0002);   // FIFO syncron latch re-enable
}
//--------------------------------------------------------------------
// Power down the radio chip
//--------------------------------------------------------------------
void MRF49XA_Power_Down(void)
{
	SPI_Command(0x8201);
}
//--------------------------------------------------------------------
// MRF49XA_Init
// Initializes the radio chip
//--------------------------------------------------------------------
void MRF49XA_Init(void){

  int delay;
  //SPI_CS = 0;
//----  configuring the RF link --------------------------------

//---- Send init cmd
  SPI_Command( FIFORSTREG );
  SPI_Command( FIFORSTREG | 0x0002);
  SPI_Command( GENCREG );
  SPI_Command( CFSREG);
  SPI_Command( PMCREG);
  SPI_Command( RXCREG);
  SPI_Command( TXCREG);
//---- antenna tunning
  SPI_Command( PMCREG | 0x0020);		// turn on tx
  DelayMS(5);
//---- end of antenna tunning
  SPI_Command( PMCREG | 0x0080);		// turn off Tx, turn on receiver
  SPI_Command( GENCREG | 0x0040);		// enable the FIFO
  SPI_Command( FIFORSTREG);
  SPI_Command( FIFORSTREG | 0x0002);	// enable syncron latch

  for(unsigned char g = 0; g < PACKET_LENGTH; g++)
  {
      packet_to_send[g] = 0x01;
      packet_received[g] = 0x01;
  }

  RF_FSEL = 1;
}

//--------------------------------------------------------------------
// MRF49XA_Send_Packet
// Sends a data packet
//--------------------------------------------------------------------
void MRF49XA_Send_Packet(unsigned char *data, unsigned char length){

	int a;
	//---- turn off receiver , enable Tx register
	SPI_Command(PMCREG);				// turn off the transmitter and receiver
	SPI_Command(GENCREG | 0x0080);		// Enable the Tx register
	//---- Packet transmission
	// Reset value of the Tx regs are [AA AA], we can start transmission
	//---- Enable Tx
	SPI_Command(PMCREG |0x0020);		// turn on tx

		SPI_CS=0;						// chip select low
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xAA);	// preamble
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0x2D);	// sync pattern 1st byte
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xD4);	// sync pattern 2nd byte
		while(!RF_SDO);
			SPI_Write16(TXBREG | length);
		#ifdef SPECTRUM_TEST
			for (a=0;1;){				// endless loop
		#else
			for (a=0;a<length;a++){				// send payload
		#endif
			while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
		#ifdef SPECTRUM_TEST
				SPI_Write16(0xAA); 		// spectrum test
		#else
				SPI_Write16(TXBREG | data[a]); // write a byte to tx register
		#endif
		}
		while(!RF_SDO){}
			SPI_Write16(TXBREG |0x00);	// write a dummy byte since the previous byte is still in buffer
		while (!RF_SDO){}				// wait for the last byte transmission end
		SPI_CS=1;						// chip select high, end transmission

	//---- Turn off Tx disable the Tx register
	SPI_Command(PMCREG | 0x0080);		// turn off Tx, turn on the receiver
	SPI_Command(GENCREG | 0x0040);		// disable the Tx register, Enable the FIFO
}


void MRF49XA_Send_Packet2()
{

	unsigned int a;
        
	//---- turn off receiver , enable Tx register
	SPI_Command(PMCREG);				// turn off the transmitter and receiver
	SPI_Command(GENCREG | 0x0080);		// Enable the Tx register
	//---- Packet transmission
	// Reset value of the Tx regs are [AA AA], we can start transmission
	//---- Enable Tx
	SPI_Command(PMCREG |0x0020);		// turn on tx

        if(packet_to_send[BYTE_VALUE] == 0) //Encoding the value if it's equal to 0
        {
            packet_to_send[BYTE_VALUE] = ZERO_VALUE_ENCODE;
        }

                SPI_CS=0;					// chip select low
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xAA);	// preamble
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0x2D);	// sync pattern 1st byte
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xD4);	// sync pattern 2nd byte
                while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
                        SPI_Write16(TXBREG | packet_to_send[BYTE_FUNCTION_ID]); // write a byte to tx register
                while(!RF_SDO); // wait for SDO, when buffer data <=8 bits
                        SPI_Write16(TXBREG | packet_to_send[BYTE_VALUE]); // write a byte to tx register
		while(!RF_SDO){}
			SPI_Write16(TXBREG |0x00);	// write a dummy byte since the previous byte is still in buffer
		while (!RF_SDO){}				// wait for the last byte transmission end
                SPI_CS=1;					// chip select high, end transmission

	//---- Turn off Tx disable the Tx register
	//SPI_Command(FIFORSTREG);   // FIFO Reset
	SPI_Command(PMCREG | 0x0080);		// turn off Tx, turn on the receiver
	SPI_Command(GENCREG | 0x0040);		// disable the Tx register, Enable the FIFO
        //SPI_Command(FIFORSTREG | 0x0002);   // FIFO syncron latch re-enable
        
}

void MRF49XA_Send_Packet4() //For 4-bytes packets //Doesn't work properly yet
{

	unsigned int a;
	//---- turn off receiver , enable Tx register
	SPI_Command(PMCREG);				// turn off the transmitter and receiver
	SPI_Command(GENCREG | 0x0080);		// Enable the Tx register
	//---- Packet transmission
	// Reset value of the Tx regs are [AA AA], we can start transmission
	//---- Enable Tx
	SPI_Command(PMCREG |0x0020);		// turn on tx

		SPI_CS=0;						// chip select low
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xAA);	// preamble
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0x2D);	// sync pattern 1st byte
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xD4);	// sync pattern 2nd byte

                        
                while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
                        SPI_Write16(TXBREG | packet_to_send[0]); // write a byte to tx register
                DelayMS(BYTES_SEND_DELAY_MS);
                while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
                        SPI_Write16(TXBREG | packet_to_send[1]); // write a byte to tx register
                DelayMS(BYTES_SEND_DELAY_MS);
                while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
                        SPI_Write16(TXBREG | packet_to_send[2]); // write a byte to tx register
                DelayMS(BYTES_SEND_DELAY_MS);
                while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
                        SPI_Write16(TXBREG | packet_to_send[3]); // write a byte to tx register
                DelayMS(BYTES_SEND_DELAY_MS);
		//while(!RF_SDO);
			//SPI_Write16(TXBREG | PACKET_LENGTH);
                /*for (a = 0; a < PACKET_LENGTH; a++)
                {				// send payload
			while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
                        SPI_Write16(TXBREG | data[a]); // write a byte to tx register
		}*/
		while(!RF_SDO){}
			SPI_Write16(TXBREG |0x00);	// write a dummy byte since the previous byte is still in buffer
		while (!RF_SDO){}				// wait for the last byte transmission end
		SPI_CS=1;						// chip select high, end transmission

	//---- Turn off Tx disable the Tx register
	//SPI_Command(FIFORSTREG);   // FIFO Reset
	SPI_Command(PMCREG | 0x0080);		// turn off Tx, turn on the receiver
	SPI_Command(GENCREG | 0x0040);		// disable the Tx register, Enable the FIFO
        //SPI_Command(FIFORSTREG | 0x0002);   // FIFO syncron latch re-enable
}
//--------------------------------------------------------------------
// MRF49XA_Receive_Packet
// Receives a data packet
// This function should be called periodically until it returns
// a valid data packet
// The return values are (see MRF49XA.h) :
// NODATA 			0			no data received
// DATA_RECEIVED 	1			data received, data capture in progress
// PACKET_RECEIVED	2			data received, data packet received ok
//--------------------------------------------------------------------
int MRF49XA_Receive_Packet( unsigned char *data, unsigned char *length){

	static unsigned char PacketLen =0;
	static unsigned char nrbytes =0;
	unsigned char bl;

	SPI_CS	=0;							// chip select low
	if(RF_SDO)
	{								// in case of FINT then read out
		SPI_CS=1;
		RF_FSEL = 0;
		if(PacketLen)
		{	// is this the first byte? go on if not
			data[(nrbytes++)]=SPI_Read();// number of bytes in payload is the first byte
			if (nrbytes >=PacketLen)
			{   // end of packet
				SPI_Command(FIFORSTREG);// reset FIFO
				RF_FSEL=1;
				// whole packet received
				*length = nrbytes;
				nrbytes=0;PacketLen =0;
				return PACKET_RECEIVED;
			}
		}
		else
		{	// the first byte received
			bl=SPI_Read();
			if((bl>0) && (bl<PAYLOAD_MAX))
			{   // check if correct number of bytes in payload
				PacketLen=bl;
				nrbytes=0;
			}
			else
			{
				RF_FSEL=1;						// bad packet length
				SPI_Command(PMCREG);			// turn off tx and rx
				SPI_Command(FIFORSTREG);		// reset FIFO
				SPI_Command(GENCREG);			// disable FIFO , Tx_latch
				SPI_Command(GENCREG | 0x0040);	// enable the FIFO
				SPI_Command(PMCREG | 0x0080);	// turn on receiver
				SPI_Command(FIFORSTREG | 0x0002);   // FIFO syncron latch re-enable
				return NODATA;
			}
		}
		RF_FSEL=1;
		return DATA_RECEIVED;
	}
	else {
		SPI_CS = 1;
		return NODATA;
	}
}



int MRF49XA_Receive_Packet2() //For 2bytes packets
{
	unsigned char bl;
        SPI_Command(PMCREG | 0x0080);		// turn off Tx, turn on the receiver
	SPI_Command(GENCREG | 0x0040);
        SPI_CS = 0;
        if(RF_SDO)
	{								// in case of FINT then read out
		RF_FSEL = 0;
                bl = SPI_Read2();
                if((bl != 0x00) && (bl >= 110)) //The byte is received and is a function number
                {
                    packet_received[BYTE_FUNCTION_ID] = bl;
                    DelayUS(BYTES_SEND_DELAY_US); //Wait for byte received guaranted
                    bl = SPI_Read2();
                    if((bl != 0x00) && (bl < 110)) ////The byte is received and is a value
                    {
                        if(bl == ZERO_VALUE_ENCODE)
                        {
                            packet_received[BYTE_VALUE] = 0;
                        } else {
                            packet_received[BYTE_VALUE] = bl;
                        }
                        //SPI_Command(FIFORSTREG);// reset FIFO
                        RF_FSEL = 1;
                        MRF49XA_Reset_Radio2();
                        SPI_CS = 1;
                        return PACKET_RECEIVED;
                    } else {
                        MRF49XA_Reset_Radio2();
                        SPI_CS = 1;
                        RF_FSEL = 1;
                        return NODATA;
                    }
                } else {
                    SPI_CS = 1;
                    RF_FSEL = 1;
                    return NODATA;
                }
        } else {
		SPI_CS = 1;
                RF_FSEL = 1;
		return NODATA;
	}
}


int MRF49XA_Receive_Packet4() //For 4bytes packets //Doesn't work properly yet
{
        unsigned char bl;
        //SPI_Command(PMCREG | 0x0080);		// turn off Tx, turn on the receiver
	//SPI_Command(GENCREG | 0x0040);
        SPI_CS = 0;
        if(RF_SDO)
	{								// in case of FINT then read out
                //SPI_CS = 1;
		RF_FSEL = 0;
                bl = SPI_Read2();
                if(bl != 0x00)
                {

                    packet_received[receivedBytesCount] = bl;
                    receivedBytesCount++;
                    if(receivedBytesCount >= PACKET_LENGTH)
                    {
                        receivedBytesCount = 0;
                        SPI_Command(FIFORSTREG);// reset FIFO
                        RF_FSEL = 1;
                        MRF49XA_Reset_Radio2();
                        //SPI_CS = 1;
                        return PACKET_RECEIVED;
                    } else {
                        bl = SPI_Read2();
                        if(bl != 0x00)
                        {
                            packet_received[receivedBytesCount] = bl;
                            receivedBytesCount++;
                            if(receivedBytesCount >= PACKET_LENGTH)
                            {
                                receivedBytesCount = 0;
                                SPI_Command(FIFORSTREG);// reset FIFO
                                RF_FSEL = 1;
                                MRF49XA_Reset_Radio2();
                                //SPI_CS = 1;
                                return PACKET_RECEIVED;
                            }
                            //SPI_Command(FIFORSTREG);// reset FIFO
                        }
                        SPI_Command(FIFORSTREG);// reset FIFO
                        RF_FSEL = 1;
                        //SPI_CS = 1;
                        return DATA_RECEIVED;
                    }
                } else {
                    SPI_CS = 1;
                    RF_FSEL = 1;
                    return NODATA;
                }
        } else {
		SPI_CS = 1;
                RF_FSEL = 1;
		return NODATA;
	}
}













































