/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/
//#include "../Headers/system.h"

#include <stdint.h>
#include <string.h>
#include <stddef.h>

#include "../Headers/usb/usb.h"
//#include "../Headers/common_config.h"

#include "../Headers/MainTasks.h"
#include "../Headers/usb_config.h"

/** VARIABLES ******************************************************/

static char buttonMessage[] = "Button pressed.\r\n";
static uint8_t readBuffer[USB_BUFFER_SIZE];
static uint8_t writeBuffer[USB_BUFFER_SIZE];

volatile USB_HANDLE USBOutHandle;
volatile USB_HANDLE USBInHandle;

static volatile unsigned char button_rocket_pressed = _FALSE;
static volatile unsigned char button_main_motor_max_pressed = _FALSE;
static volatile unsigned char button_adjust_servomotors_pressed = _FALSE;
static volatile unsigned char button_on_off_pressed = _FALSE;

unsigned char checksum = 0;
unsigned char main_motor_sp = 0;
unsigned char el_ang = 50;
unsigned char er_ang = 50;
unsigned char th_ang = 50;
unsigned char tt_ang = 50;

unsigned char main_motor_on = 0;
unsigned char adjust_mode_on = 0;

unsigned int status = 0;

unsigned char i = 0;
unsigned int curr_analog_value = 0;
unsigned long average_value = 0;

//unsigned int adc_results[100];

unsigned int numBytesRead = 0;



void APP_DeviceCDCBasicDemoInitialize()
{
    CDCInitEP();
    line_coding.bCharFormat = 0;
    line_coding.bDataBits = 8;
    line_coding.bParityType = 0;
    line_coding.dwDTERate = 9600;

}

/*void APP_DeviceCustomHIDInitialize()
{
    //initialize the variable holding the handle for the last
    // transmission
    USBInHandle = 0;

    //enable the HID endpoint
    USBEnableEndpoint(CUSTOM_DEVICE_HID_EP, USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);

    //Re-arm the OUT endpoint for the next packet
    USBOutHandle = (volatile USB_HANDLE)HIDRxPacket(CUSTOM_DEVICE_HID_EP,(uint8_t*)&readBuffer,USB_BUFFER_SIZE);
}*/

/*void CalculateChecksum()
{
    checksum = 0;
    for(i = 0; i < 8; ++i)
        checksum = checksum + (((packet_to_send[BYTE_DEVICE_ID])>>i)&0x01);
    for(i = 0; i < 8; ++i)
        checksum = checksum + (((packet_to_send[BYTE_FUNCTION_ID])>>i)&0x01);
    for(i = 0; i < 8; ++i)
        checksum = checksum + (((packet_to_send[BYTE_VALUE])>>i)&0x01);
    packet_to_send[3] = checksum;
}

void SendMainMotorSpeed4(unsigned char speed)
{
    packet_to_send[0] = TARGET_DEVICE_ID;
    packet_to_send[1] = 0xA0; //MAIN_MOTOR_SPEED;
    packet_to_send[2] = speed;
    CalculateChecksum(); //the 4-th byte of the packet
    //packet_to_send[3] = TARGET_DEVICE_ID;
    MRF49XA_Send_Packet2();
}*/

void SendMainMotorSpeed2(unsigned char speed)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = MAIN_MOTOR_SPEED;
    packet_to_send[BYTE_VALUE] = speed; //MAIN_MOTOR_SPEED;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void SendEleronsValue2(unsigned char valv)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = EL_ANGLE;
    packet_to_send[BYTE_VALUE] = valv;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
    
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = ER_ANGLE;
    packet_to_send[BYTE_VALUE] = valv;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;  
}

void SendTHValue2(unsigned char val)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = TH_ANGLE;
    packet_to_send[BYTE_VALUE] = val;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void SendTTValue2(unsigned char val)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = TT_ANGLE;
    packet_to_send[BYTE_VALUE] = val;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void SendRocketLaunch()
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = RADIO_ROCKET;
    packet_to_send[BYTE_VALUE] = 0x11;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void SendLedRed(unsigned char val)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = RADIO_RED_LED;
    packet_to_send[BYTE_VALUE] = val;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void SendLedYellow(unsigned char val)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = RADIO_YELLOW_LED;
    packet_to_send[BYTE_VALUE] = val;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void SendLedGreen(unsigned char val)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = RADIO_GREEN_LED;
    packet_to_send[BYTE_VALUE] = val;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void SendLedWhite(unsigned char val)
{
    DelayMS(PACKET_SEND_DELAY_MS);
    packet_to_send[BYTE_FUNCTION_ID] = RADIO_WHITE_LED;
    packet_to_send[BYTE_VALUE] = val;
    is_interrupt_prohibited = 1;
    MRF49XA_Send_Packet2();
    is_interrupt_prohibited = 0;
}

void ReadADJValues()
{
    is_interrupt_prohibited = 1;

    EECON1bits.WREN = 0;
    
    EEADR = 0x00;
    EECON1bits.EEPGD = 0;
    EECON1bits.RD = 1;
    Nop();
    elerons_adj = EEDATA; 

    EEADR = 0x01;
    EECON1bits.RD = 1;
    Nop();
    th_adj = EEDATA;

    EEADR = 0x02;
    EECON1bits.RD = 1;
    Nop();
    tt_adj = EEDATA;

    is_interrupt_prohibited = 0;
}

void WriteADJValues()
{
    is_interrupt_prohibited = 1;
    EECON1bits.WREN = 1;

    EEADR = 0x00;
    EEDATA = elerons_adj;
    EECON2 = 0x55;
    EECON2 = 0xAA;
    EECON1bits.WR = 1;
    Nop();

    EEADR = 0x01;
    EEDATA = th_adj;
    EECON2 = 0x55;
    EECON2 = 0xAA;
    EECON1bits.WR = 1;
    Nop();

    EEADR = 0x02;
    EEDATA = tt_adj;
    EECON2 = 0x55;
    EECON2 = 0xAA;
    EECON1bits.WR = 1;
    Nop();
    
    EECON1bits.WREN = 0;
    is_interrupt_prohibited = 0;
}
 

void ParsePacketReceived()
{
    switch(packet_received[BYTE_FUNCTION_ID])
    {
        case RADIO_DEBUG_BUTTON: //parsing data payload
        {
            LED_RED = 1;
            LED_YELLOW = 1;
            LED_GREEN = 1;
            DelayS(1);
            LED_RED = 0;
            LED_YELLOW = 0;
            LED_GREEN = 0;
            break;
        }
    }
}

void SetElerons()
{
    if((el_ang + elerons_adj) > 100)
    {
         SendEleronsValue2(100);
    } else if((el_ang + elerons_adj) < 0)
    {
         SendEleronsValue2(0);
    } else {
         SendEleronsValue2(el_ang + elerons_adj);
    }
}

void SetTH()
{
    if((th_ang + th_adj) > 100)
    {
         SendTHValue2(100);
    } else if((th_ang + th_adj) < 0)
    {
         SendTHValue2(0);
    } else {
         SendTHValue2(th_ang + th_adj);
    }
}

void SetTT()
{
    if((tt_ang + tt_adj) > 100)
    {
         SendTTValue2(100);
    } else if((tt_ang + tt_adj) < 0)
    {
         SendTTValue2(0);
    } else {
         SendTTValue2(tt_ang + tt_adj);
    }
}

void InitializeServomotors()
{
    SendEleronsValue2(el_ang + elerons_adj); //Initialize servomotors
    SendTHValue2(th_ang + th_adj);
    SendTTValue2(tt_ang + tt_adj);
}

void HandleUSBPacketReceived()
{
    //0x60 - Main Motor On/Off
    //0x61 - Main Motor Max Speed
    //0x62 - Adjust Servomotors
    //0x63 - Rocket Launch
    //0x64 - Main Motor Speed Change
    //0x65 - Elerons Adjust
    //0x66 - TH Adjust
    //0x67 - TT Adjust
    //0x68 - Turn Left
    //0x69 - Turn Up
    //0x70 - Turn Down
    //0x71 - Turn Right
    switch(readBuffer[0])
    {
        case 0x60: //Main Motor On/Off
        {
            main_motor_on = readBuffer[1];
            LED_GREEN = main_motor_on;
            SendLedGreen(main_motor_on);
            if(!main_motor_on)
            {
                main_motor_sp = 0;
                SendMainMotorSpeed2(0);
            } else {
                SendMainMotorSpeed2(main_motor_sp);
            }
            break;
        }
        case 0x61: //Main Motor Max Speed
        {
            if(main_motor_on)
            {
                if(readBuffer[1] > 0)
                {
                    LED_RED = 1;
                    SendMainMotorSpeed2(MAIN_MOTOR_MAX_SPEED_VALUE);
                    SendLedRed(1);
                } else {
                    LED_RED = 0;
                    SendMainMotorSpeed2(0);
                    SendLedRed(0);
                }
            }
            break;
        }
        case 0x62: //Adjust Servomotors
        {
            adjust_mode_on = 1 - adjust_mode_on;
            if(!adjust_mode_on) WriteADJValues();
            LED_YELLOW = adjust_mode_on;
            SendLedYellow(adjust_mode_on);
            break;
        }
        case 0x63: //Rocket Launch
        {
            packet_to_send[BYTE_FUNCTION_ID] = RADIO_ROCKET;
            packet_to_send[BYTE_VALUE] = 0x01;
            MRF49XA_Send_Packet2();
            break;
        }
        case 0x64: //Main Motor Speed Change
        {
            main_motor_sp = readBuffer[1];
            SendMainMotorSpeed2(main_motor_sp);
            break;
        }
        case 0x65: //Elerons Adjust
        {
            if(adjust_mode_on)
            {
                elerons_adj = readBuffer[1] - 20;
                SetElerons();
            }
            break;
        }
        case 0x66: //TH Adjust
        {
            if(adjust_mode_on)
            {
                th_adj = readBuffer[1] - 20;
                SetTH();
            }
            break;
        }
        case 0x67: //TT Adjust
        {
            if(adjust_mode_on)
            {
                tt_adj = readBuffer[1] - 20;
                SetTT();
            }
            break;
        }
        case 0x68: //Turn Left
        {
            if(readBuffer[1] > 0)
            {
                el_ang = 100;
                SetElerons();
                
                tt_ang = 100;
                SetTT();
            } else {
                el_ang = 50;
                SetElerons();

                tt_ang = 50;
                SetTT();
            }
            break;
        }
        case 0x69: //Turn Up
        {
            if(readBuffer[1] > 0)
            {
                tt_ang = 100;
                SetTH();
            } else {
                tt_ang = 50;
                SetTH();
            }
            break;
        }
        case 0x70: //Turn Down
        {
            if(readBuffer[1] > 0)
            {
                tt_ang = 0;
                SetTH();
            } else {
                tt_ang = 50;
                SetTH();
            }
            break;
        }
        case 0x71: //Turn Right
        {
            if(readBuffer[1] > 0)
            {
                el_ang = 0;
                SetElerons();

                tt_ang = 0;
                SetTT();
            } else {
                el_ang = 50;
                SetElerons();

                tt_ang = 50;
                SetTT();
            }
            break;
        }
        /*case 0x72: //
        {
            if(readBuffer[1] > 0)
            {
                is_usb_connected = 1;
            } else {
                is_usb_connected = 0;
            }
            break;
        }*/
    }
    //Resetting the read buffer
    for(i = 0; i < USB_BUFFER_SIZE; i++)
    {
        readBuffer[i] = 0x00;
    }
}

char MeasureADC100(char ch) ////value changed from 0 to 100
{
    //adc_results
            average_value = 0;
            for(i = 0; i < MEASURE_ADC_RETRIES; i++)
            {
                ADCON0 = ch << 2;
                ADCON0bits.ADON = 1;
                ADCON0bits.GODONE = 1;
                while(ADCON0bits.GODONE);
                curr_analog_value = ADRESH;
                curr_analog_value = curr_analog_value << 8;
                curr_analog_value = curr_analog_value + ADRESL;
                ADCON0bits.ADON = 0;
                //ADCON0=0x00;
                if(curr_analog_value > 1000) curr_analog_value = 1000;
                curr_analog_value = curr_analog_value / 10;
                average_value = average_value + curr_analog_value;
                DelayUS(MEASURE_ADC_DELAY_US);

            }
            return (average_value / MEASURE_ADC_RETRIES);
}

char MeasureADC40(char ch) //value changed from 0 to 40
{
    //adc_results
            average_value = 0;
            for(i = 0; i < MEASURE_ADC_RETRIES; i++)
            {
                ADCON0 = ch << 2;
                ADCON0bits.ADON = 1;
                ADCON0bits.GODONE = 1;
                while(ADCON0bits.GODONE);
                curr_analog_value = ADRESH;
                curr_analog_value = curr_analog_value << 8;
                curr_analog_value = curr_analog_value + ADRESL;
                ADCON0bits.ADON = 0;
                //ADCON0=0x00;
                if(curr_analog_value > 1000) curr_analog_value = 1000;
                curr_analog_value = curr_analog_value / 25;
                average_value = average_value + curr_analog_value;
                DelayUS(MEASURE_ADC_DELAY_US);

            }
            return (average_value / MEASURE_ADC_RETRIES);
}

void MainTasks()
{
    //--------------------------------------------------------------------------
    /*//Check to make sure the endpoint/buffer is free before we modify the contents
                if(!HIDTxHandleBusy(USBInHandle)) //Sending packet for HID protocol
                {
                    ToSendDataBuffer[0] = 0x81;				//Echo back to the host PC the command we are fulfilling in the first uint8_t.  In this case, the Get Pushbutton State command.
                    if(BUTTON_IsPressed(BUTTON_USB_DEVICE_HID_CUSTOM) == false)	//pushbutton not pressed, pull up resistor on circuit board is pulling the PORT pin high
                    {
                            ToSendDataBuffer[1] = 0x01;
                    }
                    else									//sw3 must be == 0, pushbutton is pressed and overpowering the pull up resistor
                    {
                            ToSendDataBuffer[1] = 0x00;
                    }
                    //Prepare the USB module to send the data packet to the host
                    USBInHandle = HIDTxPacket(CUSTOM_DEVICE_HID_EP, (uint8_t*)&ToSendDataBuffer[0],64);
                }*/
    //--------------------------------------------------------------------------
    if(USBGetDeviceState() > DEFAULT_STATE)
    {
        if(USBUSARTIsTxTrfReady() == true)
        {
            numBytesRead = getsUSBUSART(readBuffer, sizeof(readBuffer));

            if((numBytesRead == 2) && (readBuffer[0] != 0x00))
            {
                HandleUSBPacketReceived();
                //putUSBUSART(readBuffer, numBytesRead);
            }
        }
        //----------------------------------------------------------------------
        CDCTxService();
    }
    /*if(HIDRxHandleBusy(USBOutHandle) == false) //For HID protocol
    {
        USBOutHandle = HIDRxPacket(CUSTOM_DEVICE_HID_EP, (uint8_t*)&readBuffer, USB_BUFFER_SIZE);
        if(readBuffer[0] != 0x00)
        {
            HandleUSBPacketReceived();
        }
    }*/
    //--------------------------------------------------------------------------//ADC measure
    if(USBGetDeviceState() <= DEFAULT_STATE) //Unfortunately USB and ADC doesn't work together in the scheme.
    {
        if(!button_main_motor_max_pressed && main_motor_on)
        {
            curr_analog_value = MeasureADC100(0b00000010); //MeasureADC100(0b00001011); //AN2 pin
            //curr_analog_value = 100 - curr_analog_value;  //100 is maximum allowed value
            //if((curr_analog_value < (main_motor_sp - ANALOG_MIN_DIFFERENCE)) || (curr_analog_value > (main_motor_sp + ANALOG_MIN_DIFFERENCE)))
            //{
                main_motor_sp = curr_analog_value;
                SendMainMotorSpeed2(main_motor_sp);
            //}
        }
        
        //------------------------------------------------------------------
        //Elerons control measure
        curr_analog_value = MeasureADC100(0b00000000); //MeasureADC100(0b00000011); //AN0 pin
        //curr_analog_value = 100 - curr_analog_value; //100 is maximum allowed value
        //if(((curr_analog_value - el_ang)*(curr_analog_value - el_ang) < (ANALOG_MAX_DIFFERENCE*ANALOG_MAX_DIFFERENCE)) && ((curr_analog_value - el_ang)*(curr_analog_value - el_ang) >= (ANALOG_MIN_DIFFERENCE*ANALOG_MIN_DIFFERENCE)))
        //if((curr_analog_value - el_ang)*(curr_analog_value - el_ang) >= (ANALOG_MIN_DIFFERENCE*ANALOG_MIN_DIFFERENCE))
        //if((curr_analog_value < (el_ang - ANALOG_MIN_DIFFERENCE)) || (curr_analog_value > (el_ang + ANALOG_MIN_DIFFERENCE)))
        //{
            el_ang = 100 - curr_analog_value;
            SetElerons();
        //}

        //------------------------------------------------------------------
        //TH measure
        curr_analog_value = MeasureADC100(0b00000001); //MeasureADC100(0b00000111); //AN1 pin
        //curr_analog_value = 100 - curr_analog_value; //100 is maximum allowed value
        //if(((curr_analog_value - th_ang)*(curr_analog_value - th_ang) < (ANALOG_MAX_DIFFERENCE*ANALOG_MAX_DIFFERENCE)) && ((curr_analog_value - th_ang)*(curr_analog_value - th_ang) >= (ANALOG_MIN_DIFFERENCE*ANALOG_MIN_DIFFERENCE)))
        //if((curr_analog_value - th_ang)*(curr_analog_value - th_ang) < (ANALOG_MAX_DIFFERENCE*ANALOG_MAX_DIFFERENCE))
        //if((curr_analog_value < (th_ang - ANALOG_MIN_DIFFERENCE)) || (curr_analog_value > (th_ang + ANALOG_MIN_DIFFERENCE)))
        //{
            th_ang = 100 - curr_analog_value;
            SetTH();
        //}
        //------------------------------------------------------------------

        if(adjust_mode_on)
        {
                //elerons_adj can change value within [-20;20] interval
                curr_analog_value = MeasureADC100(0b00000011); //MeasureADC40(0b00001111); //AN3 pin
                //curr_analog_value = 100 - curr_analog_value; //100 is maximum allowed value
                //if(((curr_analog_value - 50) < (elerons_adj - ANALOG_MIN_DIFFERENCE)) || ((curr_analog_value - 50) > (elerons_adj + ANALOG_MIN_DIFFERENCE)))
                //if((curr_analog_value < (elerons_adj + 50 - ANALOG_MIN_DIFFERENCE)) || (curr_analog_value > (elerons_adj + 50 + ANALOG_MIN_DIFFERENCE)))
                //{
                    elerons_adj = curr_analog_value - 50;
                    SetElerons();
                //}


                //th_adj can change value within [-20;20] interval
                curr_analog_value = MeasureADC100(0b00001000); //MeasureADC40(0b00100011); //AN8 pin
                //curr_analog_value = 100 - curr_analog_value; //100 is maximum allowed value
                //if((curr_analog_value < (th_adj + 50 - ANALOG_MIN_DIFFERENCE)) || (curr_analog_value > (th_adj + 50 + ANALOG_MIN_DIFFERENCE)))
                //{
                    th_adj = curr_analog_value - 50;
                    SetTH();
                //}


                //tt_adj can change value within [-20;20] interval
                curr_analog_value = MeasureADC100(0b00000100); //MeasureADC40(0b00010011); //AN4 pin
                //curr_analog_value = 100 - curr_analog_value; //100 is maximum allowed value
                //if((curr_analog_value < (tt_adj + 50 - ANALOG_MIN_DIFFERENCE)) || (curr_analog_value > (tt_adj + 50 + ANALOG_MIN_DIFFERENCE)))
                //{
                    tt_adj = curr_analog_value - 50;
                    SetTT();
                //}

        } else {
            //TT measure
                curr_analog_value = MeasureADC100(0b00001000); //MeasureADC100(0b00100011); //AN8 pin
                //if(((curr_analog_value - tt_ang)*(curr_analog_value - tt_ang) < (ANALOG_MAX_DIFFERENCE*ANALOG_MAX_DIFFERENCE)) && ((curr_analog_value - tt_ang)*(curr_analog_value - tt_ang) >= (ANALOG_MIN_DIFFERENCE*ANALOG_MIN_DIFFERENCE)))
                //if((curr_analog_value - tt_ang)*(curr_analog_value - tt_ang) < (ANALOG_MAX_DIFFERENCE*ANALOG_MAX_DIFFERENCE))
                //if((curr_analog_value < (tt_ang - ANALOG_MIN_DIFFERENCE)) || (curr_analog_value > (tt_ang + ANALOG_MIN_DIFFERENCE)))
                //{
                    tt_ang = 100 - curr_analog_value;
                    SetTT();
                //}
        }
    }
    //--------------------------------------------------------------------------Rocket button
        if((BUTTON_ROCKET == 1) && !button_rocket_pressed)
        {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_ROCKET == 1)
            {
                button_rocket_pressed = 1; //... button is pressed
                DelayMS(PACKET_SEND_DELAY_MS);
                packet_to_send[BYTE_FUNCTION_ID] = RADIO_ROCKET;
                packet_to_send[BYTE_VALUE] = 0x01;
                MRF49XA_Send_Packet2();
            }
        } else if((BUTTON_ROCKET == 0) && button_rocket_pressed) {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_ROCKET == 0)
            {
                button_rocket_pressed = 0; //... button is left
            }
        }
    //--------------------------------------------------------------------------Adjust button
        if((BUTTON_ADJUST_SERVOMOTORS == 1) && !button_adjust_servomotors_pressed)
        {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_ADJUST_SERVOMOTORS == 1)
            {
                button_adjust_servomotors_pressed = 1; //... button is pressed
                adjust_mode_on = 1 - adjust_mode_on;
                if(!adjust_mode_on) WriteADJValues();
                LED_YELLOW = adjust_mode_on;
                SendLedYellow(adjust_mode_on);
            }
        } else if((BUTTON_ADJUST_SERVOMOTORS == 0) && button_adjust_servomotors_pressed) {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_ADJUST_SERVOMOTORS == 0)
            {
                button_adjust_servomotors_pressed = 0; //... button is left
            }
        }
    //--------------------------------------------------------------------------Main motor max button
        if((BUTTON_MAIN_MOTOR_MAX == 1) && !button_main_motor_max_pressed)
        {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_MAIN_MOTOR_MAX == 1)
            {
                button_main_motor_max_pressed = 1; //... button is pressed
                if(main_motor_on)
                {
                    LED_RED = 1;
                    SendMainMotorSpeed2(MAIN_MOTOR_MAX_SPEED_VALUE);
                    SendLedRed(1);
                }
            }
        } else if((BUTTON_MAIN_MOTOR_MAX == 0) && button_main_motor_max_pressed)
        {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_MAIN_MOTOR_MAX == 0)
            {
                button_main_motor_max_pressed = 0; //... button is left
                if(main_motor_on) 
                {
                    LED_RED = 0;
                    SendMainMotorSpeed2(main_motor_sp);
                    SendLedRed(0);
                }
            }
        }
    //--------------------------------------------------------------------------Main motor on/off button
        if((BUTTON_ON_OFF_MAIN_MOTOR == 1) && !button_on_off_pressed)
        {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_ON_OFF_MAIN_MOTOR == 1)
            {
                button_on_off_pressed = 1; //... button is pressed
                main_motor_on = 1 - main_motor_on;
                LED_GREEN = main_motor_on;
                SendLedGreen(main_motor_on);
                if(!main_motor_on) 
                {
                    main_motor_sp = 0;
                    SendMainMotorSpeed2(0);
                    LED_RED = 0;
                    SendLedRed(0);
                    SendMainMotorSpeed2(0);
                }
            }
        } else if((BUTTON_ON_OFF_MAIN_MOTOR == 0) && button_on_off_pressed)
        {
            DelayMS(BUTTON_CHECK_DELAY_MS);
            if(BUTTON_ON_OFF_MAIN_MOTOR == 0)
            {
                button_on_off_pressed = 0; //... button is left                
            }
        }
    //--------------------------------------------------------------------------Receive job
        //DelayMS(10);
        status = MRF49XA_Receive_Packet2();
        if(status == PACKET_RECEIVED) ParsePacketReceived();
}


#if defined(__XC8)
void interrupt SYS_InterruptHigh(void)
{
    //if(!is_interrupt_prohibited)
    //{
        #if defined(USB_INTERRUPT)
            USBDeviceTasks();
            //is_usb_connected = 1;
        #endif
    //}
}
#endif


