﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;

namespace USB_CDC
{
    public partial class MainForm : Form
    {
        static SerialPort mainPort = null;
        public static Mutex mut = new Mutex();
        private Thread receivedHandler;

        public List<string> listReceived;

        //delegate to log received text
        public delegate void AddDataDelegate(String myString);
        public AddDataDelegate myDelegate;

        private char[] command = { (char)0xAB, (char)0x00 };
        //private byte[] command = { 0xAB, 0x00, 0x00, 0x00 };

        public bool main_motor_on = false;
        public bool adjust_mode = false;

        public bool button_main_motor_on_off_pressed = false;
        public bool button_main_motor_max_pressed = false;
        public bool button_adjust_pressed = false;
        public bool button_rocket_pressed = false;

        public int old_trackbar1_value = 0;

        Thread th = null;

        public MainForm()
        {
            InitializeComponent();
            this.FormClosing += new FormClosingEventHandler(MainForm_FormClosing);
            this.myDelegate = new AddDataDelegate(AddDataMethod);
            listReceived = new List<string>();
            this.button5.Focus();
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------

        private void threadHandler()
        {
            if (listReceived.Count > 0)
            {
                foreach (string item in listReceived)
                {
                    LogReceived(item);
                    listReceived.Remove(item);
                }
            }
            Thread.Sleep(500);
        }

        static byte[] GetBytes(string str)
        {
            byte[] bytes = new byte[str.Length * sizeof(char)];
            System.Buffer.BlockCopy(str.ToCharArray(), 0, bytes, 0, bytes.Length);
            return bytes;
        }

        static string GetString(byte[] bytes)
        {
            char[] chars = new char[bytes.Length / sizeof(char)];
            System.Buffer.BlockCopy(bytes, 0, chars, 0, bytes.Length);
            return new string(chars);
        }

        public void writePort(string str)
        {
            try
            {
                mainPort.Write(System.Text.Encoding.ASCII.GetBytes(str), 0, str.Length);
                LogSent(str);
            }
            catch (Exception ex)
            {
            }
        }

        /*public void writePort(long val)
        {
            try
            {
                char[] chars = {'1', '1', '1', '1'};
                chars[0] = (char)(171); //(char)((val >> 24) & 0x000000FF);
                chars[1] = (char)((val >> 16) & 0x000000FF);
                chars[2] = (char)((val >> 8) & 0x000000FF);
                chars[3] = (char)(val & 0x000000FF);
                string s = new string(chars);
                mainPort.Write(System.Text.Encoding.ASCII.GetBytes(s), 0, s.Length);
                //LogSent("0x" + val.ToString("X8"));
                LogSent("0x" + s);
            }
            catch (Exception ex)
            {
            }
        }*/

        public void AddDataMethod(String myString)
        {
            LogReceived("---" + myString);
            //LogReceived("---" + "0x" + Convert.ToInt32(myString).ToString("X8"));
        }

        public void LogReceived(string str)
        {
                //mut.WaitOne();
                //richTextBox1.Text = richTextBox1.Text + str + "\n";
                //mut.ReleaseMutex();
        }

        public void LogSent(string str)
        {

        }

        //----------------------------------------------------------------------------------------------------------------------------------------------

        public void OpenPort(string portname)
        {
            label1.Text = "Conencted to: " + portname;
            mainPort = new SerialPort(portname, 9600, Parity.None, 8, StopBits.One);
            mainPort.DataReceived += new SerialDataReceivedEventHandler(DataReceivedHandler);
            //mainPort.ReadBufferSize = 4096; mainPort.WriteBufferSize = 4096;
            //receivedHandler = new Thread(threadHandler);
            //receivedHandler.Start();
            //label4.Text = "Length: " + textBox1.Text.Length.ToString();
            mainPort.Open();
        }

        private void MainForm_FormClosing(Object sender, FormClosingEventArgs e)
        {
            try
            {
                mainPort.Close();
                mainPort.Dispose();
            }
            catch(Exception ex)
            {
            }
            Environment.Exit(0);
        }

        private void DoUpdate(object s, EventArgs e)
        {
            LogReceived(mainPort.ReadExisting());
        }

        private void DataReceivedHandler(object sender, SerialDataReceivedEventArgs e)
        {
            this.BeginInvoke(this.myDelegate, new object[] { mainPort.ReadExisting() });
        }



        //0x60 - Main Motor On/Off
        //0x61 - Main Motor Max Speed
        //0x62 - Adjust Servomotors
        //0x63 - Rocket Launch
        //0x64 - Main Motor Speed Change
        //0x65 - Elerons Adjust
        //0x66 - TH Adjust
        //0x67 - TT Adjust
        //0x68 - Turn Left
        //0x69 - Turn Up
        //0x70 - Turn Down
        //0x71 - Turn Right


        private void trackBar1_ValueChanged(object sender, System.EventArgs e) //Main motor speed
        {
            if (main_motor_on)
            {
                command[0] = (char)0x64;
                command[1] = (char)(trackBar1.Value);
                writePort(new string(command));
            }
        }

        private void trackBar4_ValueChanged(object sender, System.EventArgs e) //Adjust elerons
        {
            if (adjust_mode)
            {
                command[0] = (char)0x65;
                command[1] = (char)(trackBar4.Value);
                writePort(new string(command));
            }
        }

        private void trackBar3_ValueChanged(object sender, System.EventArgs e) //Adjust TH
        {
            if (adjust_mode)
            {
                command[0] = (char)0x66;
                command[1] = (char)(trackBar3.Value);
                writePort(new string(command));
            }
        }

        private void trackBar2_ValueChanged(object sender, System.EventArgs e) //Adjust TT
        {
            if (adjust_mode)
            {
                command[0] = (char)0x67;
                command[1] = (char)(trackBar2.Value);
                writePort(new string(command));
            }
        }


        private void button5_Click(object sender, EventArgs e) //Main Motor On/Off
        {
            main_motor_on = !main_motor_on;
            if (main_motor_on)
            {
                button5.BackColor = Color.LightGreen;
                button6.Enabled = true;
                trackBar1.Enabled = true;
            }
            else
            {
                button5.BackColor = SystemColors.Control;
                button6.Enabled = false;
                trackBar1.Enabled = false;
            }
            command[0] = (char)0x60;
            command[1] = (char)((main_motor_on) ? 0x01 : 0x00);
            writePort(new string(command));
        }

        //------------------------------------------------------------------------------------------------
        private void button6_Click(object sender, EventArgs e) //Main Motor Max Speed
        {
            /*if (main_motor_on)
            {
                command[0] = (char)0x61;
                command[1] = (char)((button_main_motor_max_pressed) ? 0x01 : 0x00);
                writePort(new string(command));
            }*/
        }

        private void button6_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) 
        {
            if (main_motor_on)
            {
                button_main_motor_max_pressed = true;
                button6.BackColor = Color.LightCoral;
                //old_trackbar1_value = trackBar1.Value;
                //trackBar1.Value = 0x48;
                command[0] = (char)0x61;
                command[1] = (char)(0x48);
                writePort(new string(command));
            }
        }

        private void button6_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) 
        {
            if (main_motor_on)
            {
                button_main_motor_max_pressed = false;
                button6.BackColor = SystemColors.Control;
                //trackBar1.Value = old_trackbar1_value;
                command[0] = (char)0x61;
                command[1] = (char)(0x00);
                writePort(new string(command));
            }
        }
        //------------------------------------------------------------------------------------------------

        private void button7_Click(object sender, EventArgs e) //Adjust Servomotors
        {
            adjust_mode = !adjust_mode;
            if (adjust_mode)
            {
                button7.BackColor = Color.Gold;
                trackBar2.Enabled = true;
                trackBar3.Enabled = true;
                trackBar4.Enabled = true;
            }
            else
            {
                button7.BackColor = SystemColors.Control;
                trackBar2.Enabled = false;
                trackBar3.Enabled = false;
                trackBar4.Enabled = false;
            }
            command[0] = (char)0x62;
            command[1] = (char)((adjust_mode)  ? 0x01 : 0x00);
            writePort(new string(command));
        }

        private void button8_Click(object sender, EventArgs e) //Rocket Launch
        {
            command[0] = (char)0x63;
            command[1] = (char)(0x01);
            writePort(new string(command));
            button8.BackColor = Color.LightSkyBlue;
            th = new Thread(rocket_cease_launch);
            th.Start();
        }

        private void rocket_cease_launch()
        {
            Thread.Sleep(1000);
            button8.BackColor = SystemColors.Control;
        }




        private void MainForm_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (e.KeyCode == Keys.D)
            {
                command[0] = (char)0x71;
                command[1] = (char)(0x01);
                writePort(new string(command));
                button12.BackColor = SystemColors.MenuHighlight;
            }
            if (e.KeyCode == Keys.A)
            {
                command[0] = (char)0x68;
                command[1] = (char)(0x01);
                writePort(new string(command));
                button9.BackColor = SystemColors.MenuHighlight;
            }
            if (e.KeyCode == Keys.W)
            {
                command[0] = (char)0x69;
                command[1] = (char)(0x01);
                writePort(new string(command));
                button10.BackColor = SystemColors.MenuHighlight;
            }
            if (e.KeyCode == Keys.S)
            {
                command[0] = (char)0x70;
                command[1] = (char)(0x01);
                writePort(new string(command));
                button11.BackColor = SystemColors.MenuHighlight;
            }

            if (e.KeyCode == Keys.CapsLock) //Main Motor On Off
            {
                main_motor_on = !main_motor_on;
                if (main_motor_on)
                {
                    button5.BackColor = Color.LightGreen;
                    button6.Enabled = true;
                    trackBar1.Enabled = true;
                }
                else
                {
                    button5.BackColor = SystemColors.Control;
                    button6.Enabled = false;
                    trackBar1.Enabled = false;
                }
                command[0] = (char)0x60;
                command[1] = (char)((main_motor_on) ? 0x01 : 0x00);
                writePort(new string(command));
            }
            if (e.KeyCode == Keys.Back) //Adjust mode
            {
                adjust_mode = !adjust_mode;
                if (adjust_mode)
                {
                    button7.BackColor = Color.Gold;
                    trackBar2.Enabled = true;
                    trackBar3.Enabled = true;
                    trackBar4.Enabled = true;
                }
                else
                {
                    button7.BackColor = SystemColors.Control;
                    trackBar2.Enabled = false;
                    trackBar3.Enabled = false;
                    trackBar4.Enabled = false;
                }
                command[0] = (char)0x62;
                command[1] = (char)((adjust_mode) ? 0x01 : 0x00);
                writePort(new string(command));
            }
            if ((e.KeyCode == Keys.P) && (main_motor_on)) //Main Motor Max
            {
                button6.BackColor = Color.LightCoral;
                command[0] = (char)0x61;
                command[1] = (char)(0x48);
                writePort(new string(command));
            }
            if (e.KeyCode == Keys.R)
            {
                command[0] = (char)0x63;
                command[1] = (char)(0x01);
                writePort(new string(command));
                button8.BackColor = Color.LightSkyBlue;
                th = new Thread(rocket_cease_launch);
                th.Start();
            }
        }

        private void MainForm_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (e.KeyCode == Keys.D)
            {
                command[0] = (char)0x71;
                command[1] = (char)(0x00);
                writePort(new string(command));
                button12.BackColor = SystemColors.Control;
            }
            if (e.KeyCode == Keys.A)
            {
                command[0] = (char)0x68;
                command[1] = (char)(0x00);
                writePort(new string(command));
                button9.BackColor = SystemColors.Control;
            }
            if (e.KeyCode == Keys.W)
            {
                command[0] = (char)0x69;
                command[1] = (char)(0x00);
                writePort(new string(command));
                button10.BackColor = SystemColors.Control;
            }
            if (e.KeyCode == Keys.S)
            {
                command[0] = (char)0x70;
                command[1] = (char)(0x00);
                writePort(new string(command));
                button11.BackColor = SystemColors.Control;
            }
            if ((e.KeyCode == Keys.P) && (main_motor_on)) //Main Motor Max
            {
                button6.BackColor = SystemColors.Control;
                command[0] = (char)0x61;
                command[1] = (char)(trackBar1.Value);
                writePort(new string(command));
            }
            if ((e.KeyCode == Keys.OemOpenBrackets) && (main_motor_on)) //Main Motor speed -
            {
                if (trackBar1.Value < 10)
                {
                    trackBar1.Value = 0;
                }
                else
                {
                    trackBar1.Value = trackBar1.Value - 10;
                }
            }
            if ((e.KeyCode == Keys.OemCloseBrackets) && (main_motor_on)) //Main Motor speed +
            {
                if (trackBar1.Value > 90)
                {
                    trackBar1.Value = 100;
                }
                else
                {
                    trackBar1.Value = trackBar1.Value + 10;
                }
            }
        }

        //------------------------------------------------------------------------------------------------
        private void button9_Click(object sender, EventArgs e) //Left
        {
        }

        private void button9_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) //Left
        {
            command[0] = (char)0x68;
            command[1] = (char)(0x01);
            writePort(new string(command));
            button9.BackColor = SystemColors.MenuHighlight;
        }

        private void button9_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) //Left
        {
            command[0] = (char)0x68;
            command[1] = (char)(0x00);
            writePort(new string(command));
            button9.BackColor = SystemColors.Control;
        }

        //------------------------------------------------------------------------------------------------
        private void button10_Click(object sender, EventArgs e) //Up
        {
        }

        private void button10_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) //Up
        {
            command[0] = (char)0x69;
            command[1] = (char)(0x01);
            writePort(new string(command));
            button10.BackColor = SystemColors.MenuHighlight;
        }

        private void button10_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) //Up
        {
            command[0] = (char)0x69;
            command[1] = (char)(0x00);
            writePort(new string(command));
            button10.BackColor = SystemColors.Control;
        }

        //------------------------------------------------------------------------------------------------
        private void button11_Click(object sender, EventArgs e) //Down
        {
        }

        private void button11_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) //Down
        {
            command[0] = (char)0x70;
            command[1] = (char)(0x01);
            writePort(new string(command));
            button11.BackColor = SystemColors.MenuHighlight;
        }

        private void button11_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) //Down
        {
            command[0] = (char)0x70;
            command[1] = (char)(0x00);
            writePort(new string(command));
            button11.BackColor = SystemColors.Control;
        }

        //------------------------------------------------------------------------------------------------
        private void button12_Click(object sender, EventArgs e) //Right
        {
        }

        private void button12_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) //Right
        {
            command[0] = (char)0x71;
            command[1] = (char)(0x01);
            writePort(new string(command));
            button12.BackColor = SystemColors.MenuHighlight;
        }

        private void button12_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) //Right
        {
            command[0] = (char)0x71;
            command[1] = (char)(0x00);
            writePort(new string(command));
            button12.BackColor = SystemColors.Control;
        }
        //------------------------------------------------------------------------------------------------
    }
}
